/* symtab.c   R1 compiler
	P. Chongstitvatana					19 Aug 97
	modify for project A1  				21 Oct 2002
	modify (add field lv) for Som-C  	27 Dec 2003
*/

#include "compile.h"

#define		PRIVATE		static


/* ==========  Interface =============

// search all symtab, return index, 0 if not found
int search(char *name)

// insert local symbol at the end of table
int enterLocal(char *name)

// insert global symbol at the end of table
int enterGlobal(char *name, int type, int ref, int arg)

// get attribute at index
int getRef(int idx)
int getArg(int idx)
int getLv(int idx)
int getType(int idx)

// set attribute at index
void setRef(int index, int ref)
void setArg(int idx, int arg)
void setLv(int idx, int lv)

// find name of given type and ref, use in listing
char *findName(int type, int ref)

// find name of given index, use in listing
char *getName(int nm)

void dumpSymTab()

 ====================================  */

/* symbols start from index 1, reserve index 0 */
PRIVATE sym_entry symtab[TABLESIZE+LOCALSIZE];
PRIVATE int nsym = 0;

int lv = 0;

/* sequential search ret index to symtab[] if found, 0 if not */
PRIVATE int searchLocal(char *name){
	int i, found;
	found = 0;
	for(i=TABLESIZE+1;i<=TABLESIZE+lv;i++)
		if( eqs(symtab[i].name, name) ){
		   found = 1;
		   break;
		}
	return found ? i : 0;
}

/* insert local symbol at the end of table */
int enterLocal(char *name){
	int a;
	lv++;
	if( lv >= LOCALSIZE ) seterror("local symbol table full");
	a = TABLESIZE + lv;
	strcpy(symtab[a].name, name);
	symtab[a].type = tyLOCAL;
	symtab[a].ref = lv;
	symtab[a].arg = 0;
	symtab[a].lv = 0;
	return a;
}

PRIVATE int searchGlobal(char *name){
	int i;
	for(i=1;i<=nsym;i++)
		if( eqs(symtab[i].name, name) ) break;
	return i > nsym ? 0 : i;
}

/* insert global symbol at the end of table */
int enterGlobal(char *name, int type, int ref, int arg){
	int a;
	nsym++;
	if( nsym >= TABLESIZE )
		seterror("global symbol table full");
	a = nsym;
	strcpy(symtab[a].name, name);
	symtab[a].type = type;
	symtab[a].ref = ref;
	symtab[a].arg = arg;
	symtab[a].lv = 0;
	return a;
}

/* search all symtab, return index, 0 if not found */
int search(char *name){
	int a;
	a = searchLocal(name);
	if( a != 0 ) return a;
	a = searchGlobal(name);
	return a;
}

int getRef(int index){ return symtab[index].ref;}
void setRef(int index, int ref){symtab[index].ref = ref;}

int getArg(int idx){ return symtab[idx].arg; }
void setArg(int idx, int arg){ symtab[idx].arg = arg; }

int getLv(int idx){ return symtab[idx].lv; }
void setLv(int idx, int lv){ symtab[idx].lv = lv; }

int getType(int idx){ return symtab[idx].type; }
void setType(int index, int type){symtab[index].type = type;}

// output symbol table to file
// if lv = 0, no local will be outputed
void dumpSymTab(FILE *FO){
	int i;
	sym_entry *s;
	fprintf(FO,"%d\n",nsym);
	for(i=1;i<=nsym;i++){
		s = &symtab[i];
		if( s->type != tyENUM )
			fprintf(FO,"%s %d %d %d %d\n",
		  	  s->name, s->type, s->ref, s->arg, s->lv);
	}
	for(i=TABLESIZE+1;i<=TABLESIZE+lv; i++){
		s = &symtab[i];
		fprintf(FO,"%s %d %d\n",
		  s->name, s->type, s->ref);
	}
}

char *findName(int type, int ref){
	int i;
	for(i=1; i<=nsym; i++)
		if( (symtab[i].type == type) && (symtab[i].ref == ref) )
			return symtab[i].name;
	return NULL;
}

char *getName(int nm){
	return symtab[nm].name;
}

// find index from a reference
int getIdx(int ref){
	int i;
	for(i=1; i<=nsym; i++)
		if(symtab[i].ref == ref)
			return i;
	return 0;
}

